#include <header.h>

void config_cleanup( struct config_t *config )
{
	if ( config->port != NULL )
		free( config->port );
	if ( config->file_config != NULL )
		free( config->file_config );
	if ( config->directory_index_file != NULL )
		free( config->directory_index_file );
	if ( config->vhost != NULL )
		free( config->vhost );
	xmlCleanupParser();
}

int parse_config( const int argc, char * const argv[], struct config_t *config )
{
	int		c;		//getopt
	extern char	*optarg;	//getopt
	extern int	optind;		//getopt

	extern int	terminate;	//declare in main.c

	xmlDocPtr	doc;
	xmlNode		*node_server;
	xmlNode		*subnode;
	xmlNode		*subsubnode;
	char		*value;
	int		value_len;


	optind = 1;

	memset( config, 0, sizeof(struct config_t) );

	config->file_config = malloc( sizeof(char) * 1024 );
	strncpy( config->file_config, DEFAULT_CONFIG_FILE_CONFIG, 1024 );

	while ( (c = getopt(argc, argv, "c:h")) > 0 )
	{
		switch ( c )
		{
			case 'c':
				config->file_config = strncpy( config->file_config, optarg, 1024 );
			break;
			case 'h':
				printf("Usage: midware1 [-c configfile]\n");
				terminate = TRUE;
				free( config->file_config );
				return 0;
			break;
		}
	}

	doc = xmlParseFile(config->file_config);

	if ( doc == NULL )
	{
		free( config->file_config );
		return -1;
	}

	/* server configuration begin */
	node_server = xml_find_doc_child_by_tag( doc, "Server" );

	if ( node_server == NULL )
	{
		config_cleanup( config );
		return -1;
	}

	config->listen_backlog =	xml_get_config_int_value( node_server, "ListenBacklog", DEFAULT_CONFIG_LISTEN_BACKLOG );
	config->timeout =		xml_get_config_int_value( node_server, "TimeOut", DEFAULT_CONFIG_TIMEOUT );
	config->thread_pool_size =	xml_get_config_int_value( node_server, "ThreadPoolSize", DEFAULT_CONFIG_THREAD_POOL_SIZE );
	if ( config->thread_pool_size < 4 )
		config->thread_pool_size = 4;

	value =				xml_get_config_char_value( node_server, "BufferSize", DEFAULT_CONFIG_DEFAULT_BUF_SIZE );
	value_len = strlen( value );
	switch ( value[value_len-1] )
	{
		case 'k':
		case 'K':
			config->default_buffer_size = atoi( value ) * 1024;
		break;
		case 'm':
		case 'M':
			config->default_buffer_size = atoi( value ) * 1024 * 1024;
		break;
		case 'g':
		case 'G':
			config->default_buffer_size = atoi( value ) * 1024 * 1024 * 1024;
		break;
		default:
			config->default_buffer_size = atoi( value );
		break;
	}

	if ( config->default_buffer_size <= 0 )
	{
		config_cleanup( config );
		return -1;
	}

	config->file_mime =		xml_get_config_char_value( node_server, "MimeFile", DEFAULT_CONFIG_FILE_MIME );

	config->server_user =		xml_get_config_char_value( node_server, "ServerUser", DEFAULT_CONFIG_SERVER_USER );
	config->document_root =		xml_get_config_char_value( node_server, "DocumentRoot", DEFAULT_CONFIG_DOCUMENT_ROOT );
	config->default_charset =	xml_get_config_char_value( node_server, "DefaultCharSet", DEFAULT_CONFIG_DEFAULT_CHARSET );

	/* port setting start */
	config->port_len = 0;
	subnode = xml_find_child_by_tag( node_server, "Listen" );
	if ( subnode == NULL )
	{
		config->port_len = 1;
		config->port = malloc( sizeof( int ) );
		config->port[0] = DEFAULT_CONFIG_PORT;
	}
	else
	{
		subsubnode = xml_find_child_by_tag( subnode, "Port" );
		config->port_len = 1;
		config->port = malloc( sizeof( int ) );
		if ( subsubnode == NULL )
			config->port[0] = DEFAULT_CONFIG_PORT;
		else
		{
			config->port[0] = atoi( xml_get_config_char_value( subsubnode, NULL, NULL ) );
			for ( subsubnode = xml_find_next_by_tag( subsubnode, "Port" ); subsubnode; subsubnode = xml_find_next_by_tag( subsubnode, "Port" ) )
			{
				config->port = realloc( config->port, sizeof(int) * (config->port_len + 1) );
				config->port[config->port_len] = atoi(xml_get_config_char_value( subsubnode, NULL, NULL ));
				config->port_len++;
			}
		}
	}
	/* port setting end */

	/* document_root & directory index setting start */
	config->directory_index_len = 0;
	subnode = xml_find_child_by_tag( node_server, "DirectoryIndex" );
	if ( subnode == NULL )
	{
		config->directory_index_len = 1;
		config->directory_index_file = malloc( sizeof(char *) * 1 );
		config->directory_index_file[0] = DEFAULT_CONFIG_DIRECTORY_INDEX;
	}
	else
	{
		subsubnode = xml_find_child_by_tag( subnode, "Filename" );
		config->directory_index_len = 1;
		config->directory_index_file = malloc( sizeof(char *) * 1 );
		if ( subsubnode == NULL )
			config->directory_index_file[0] = DEFAULT_CONFIG_DIRECTORY_INDEX;
		else
		{
			config->directory_index_file[0] = xml_get_config_char_value( subsubnode, NULL, NULL );
			for ( subsubnode = xml_find_next_by_tag( subsubnode, "Filename" ); subsubnode; subsubnode = xml_find_next_by_tag( subsubnode, "Filename" ) )
			{
				config->directory_index_file = realloc( config->directory_index_file, sizeof(char *) * (config->directory_index_len + 1) );
				config->directory_index_file[config->directory_index_len] = xml_get_config_char_value( subsubnode, NULL, NULL );
				config->directory_index_len++;
			}
		}
	}
	/* document_root & directory index setting end */

	/* virtual host setting start */
	config->vhost_len = 0;
	config->vhost = malloc( sizeof( struct vhost_t ) );
	for ( subnode = xml_find_child_by_tag( node_server, "VirtualHost" ); subnode; subnode = xml_find_next_by_tag( subnode, "VirtualHost" ) )
	{
		config->vhost = realloc( config->vhost, sizeof( struct vhost_t ) * (config->vhost_len + 1) );
		config->vhost[config->vhost_len].host =			xml_get_config_char_value( subnode, "Host", NULL );
		config->vhost[config->vhost_len].port =			xml_get_config_int_value( subnode, "Port", config->port[0] );
		config->vhost[config->vhost_len].document_root =	xml_get_config_char_value( subnode, "DocumentRoot", NULL );
		config->vhost[config->vhost_len].file_log =		xml_get_config_char_value( subnode, "LogFile", NULL );
		if ( config->vhost[config->vhost_len].host == NULL )
			continue;
		else
			config->vhost[config->vhost_len].host_len = strlen( config->vhost[config->vhost_len].host );

		config->vhost_len++;
	}
	/* virtual host setting end */

	return 0;
}

